@@ -353,7 +353,7 @@ looper_locked_region(STROB * buf, int vlv)
 
 		"				shls_bashin2 \"" SWBIS_TS_remove_catalog_entry "\"\n"
 		"				sw_retval=$?\n"
-		"				case $sw_retval in 0) $sh_dash_s " SWBIS_TS_remove_catalog_entry ";; *) shls_false_;; esac\n"
+		"				case $sw_retval in 0) $sh_dash_s " SWBIS_TS_remove_catalog_entry ";; *) sleep 2; shls_false_;; esac\n"
 		"				sw_retval=$?\n"
 		"				swexec_status=$sw_retval\n"
 
@@ -723,6 +723,12 @@ swremove_write_source_copy_script2(GB * G,
 		"sw_retval=$?\n"
 		"swexec_status=$sw_retval\n"
 
+		"shls_bashin2 \"" SWBIS_TS_Do_nothing "\"\n"
+		"sw_retval=$?\n"
+		"case $sw_retval in 0) $sh_dash_s " SWBIS_TS_Do_nothing ";; *) shls_false_;; esac\n"
+		"sw_retval=$?\n"
+		"swexec_status=$sw_retval\n"
+
 		/* Loop over the selections */
 		"		shls_looper \"$sw_retval\"\n"
 		"		sw_retval=$?\n"
@@ -974,10 +980,10 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 
 	e = swicat_e_create();
 	ret = swicat_e_open_entry_tarball(e, ifd);
-	if (ret < 0) {
+	if (ret != 0) {
 		sw_e_msg(G, "error opening catalog entry tarball, ret=%d\n", ret);
 		rstatus = 1;
-		retval = -1;  /* protocol internal error */
+		retval = -2;  /* protocol internal error */
 		looper_abort(G, swicol, ofd);
 		goto error_out;
 	} else if (ret == SWICAT_RETVAL_NULLARCHIVE) {
@@ -1007,7 +1013,7 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	ret = swicat_e_reset_fd(e);
 	if (ret < 0) {
 		rstatus = 2;
-		retval = -1;
+		retval = -3;
 		sw_e_msg(G, "error reseting swicat_e object, ret=%d\n", ret);
 		looper_abort(G, swicol, ofd);
 		goto error_out;
@@ -1020,6 +1026,9 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	swgpg = swgpg_create();
 	if (G->devel_verboseM)
 		SWLIB_INFO2("NOPEN=%d", uxfio_uxfio_get_nopen());
+
+	E_DEBUG("");
+
 	ret = swicat_e_verify_gpg_signature(e, swgpg);
 	if (ret != 0) {
 		rstatus = 3;
@@ -1035,7 +1044,7 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 		/* Bad signature or not enough good signatures
 		   Can't use the blob to make a removal file list
 		   because were assuming its tainted. */
-		rstatus = 4;
+		rstatus = SWP_RP_STATUS_NO_INTEGRITY;
 		retval = 0;
 		do_skip_entry = 1;
 	} else {
@@ -1054,14 +1063,15 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 		   handle NULL gracefully */
 		do_skip_entry = 1;
 		sw_e_msg(G, "catalog read error, possible permission denied: %s/%s\n", target_path, epath);
+	} else {
+		swi->swi_pkgM->catalog_entryM = strdup(epath); /* FIXME, this must set explicitly */
+		swi->swi_pkgM->target_pathM = strdup(target_path); /* FIXME, this must set explicitly */
+		swi->optaM = G->optaM;
 	}
-	
-	if (swi) swi->swi_pkgM->catalog_entryM = strdup(epath); /* FIXME, this must set explicitly */
-	if (swi) swi->swi_pkgM->target_pathM = strdup(target_path); /* FIXME, this must set explicitly */
 
 	if (G->devel_verboseM)
 		SWLIB_INFO2("NOPEN=%d", uxfio_uxfio_get_nopen());
-	if (swi) swi->optaM = G->optaM;
+
 	if (G->devel_verboseM)
 		SWLIB_INFO2("NOPEN=%d", uxfio_uxfio_get_nopen());
 
@@ -1090,10 +1100,9 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	*p_rp_status = ret;
 	E_DEBUG2("rp_status is %d", ret);
 	if (ret) {
-		/* looper_abort(G, swicol, ofd); */
 		rstatus = 6;
-		retval = 0;
-		do_skip_entry = 1;
+		retval = -4;
+		goto error_out;
 	}
 	if (G->g_do_debug_events)
 		swicol_show_events_to_fd(swicol, STDERR_FILENO, -1);
@@ -1104,7 +1113,9 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	*p_rp_status = ret;
 	E_DEBUG2("rp_status is %d", ret);
 	if (ret) {
-		looper_abort(G, swicol, ofd);
+		rstatus = -1;
+		retval = -5;
+		goto error_out;
 	}
 
 	if (G->g_do_debug_events)
@@ -1116,13 +1127,10 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 		file_list = NULL;
 
 	if (swi && file_list == NULL) {
+		/* empty package, no files */
 		rstatus = 7;
 		retval = 0;
-		do_skip_entry = 1;
-		/* 
-		looper_abort(G, swicol, ofd);
 		goto error_out;
-		*/
 	}
 	
 	if (
@@ -1147,9 +1155,10 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 			SWICOL_TL_8,
 			SW_SUCCESS);
 		if (ret != 0) {
-			looper_abort(G, swicol, ofd);
+			E_DEBUG("");
+			rstatus = -2;
+			retval = -8;
 			goto error_out;
-			retval = -1;
 		}
 	} else {
 		ret = swpl_unpack_catalog_tarfile(G, swi, ofd,
@@ -1158,9 +1167,10 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 			0 /*alt_catalog_root*/,
 			G->g_swi_event_fd);
 		if (ret != 0) {
-			looper_abort(G, swicol, ofd);
+			E_DEBUG("");
+			rstatus = -3;
+			retval = -9;
 			goto error_out;
-			retval = -1;
 		}
 	}
 
@@ -1179,9 +1189,10 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 			SWICOL_TL_8,
 			SW_SUCCESS);
 		if (ret != 0) {
-			looper_abort(G, swicol, ofd);
+			E_DEBUG("");
+			rstatus = -4;
+			retval = -10;
 			goto error_out;
-			retval = -1;
 		}
 	} else {
 		/*
@@ -1235,9 +1246,9 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 			SWICOL_TL_8,
 			SW_SUCCESS);
 		if (ret != 0) {
-			looper_abort(G, swicol, ofd);
+			rstatus = -7;
+			retval = -10;
 			goto error_out;
-			retval = -1;
 		}
 	} else {
 		E_DEBUG("remove files");
@@ -1245,9 +1256,8 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 		if (ret < 0) {
 			/* internal error */
 			E_DEBUG("remove files: internal error");
-			rstatus = 8;
-			retval = -1;
-			looper_abort(G, swicol, ofd);
+			rstatus = -8;
+			retval = -11;
 			goto error_out;
 		} else if (ret > 0) {
 			/* script error */
@@ -1271,9 +1281,9 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 			SWICOL_TL_8,
 			SW_SUCCESS);
 		if (ret != 0) {
-			looper_abort(G, swicol, ofd);
+			rstatus = -10;
+			retval = -12;
 			goto error_out;
-			retval = -1;
 		}
         } else {
                 /* TS_Catalog_dir_remove */
@@ -1283,9 +1293,9 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 			0 /*alt_catalog_root*/,
 			G->g_swi_event_fd);
 		if (ret != 0) {
-			looper_abort(G, swicol, ofd);
+			rstatus = -11;
+			retval = -13;
 			goto error_out;
-			retval = -1;
 		}
 	}
 
@@ -1301,20 +1311,20 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 			SWICOL_TL_8,
 			SW_SUCCESS);
 		if (ret != 0) {
-			retval = -1;
+			retval = -13;
 		}
 	} else {
 		ret = swpl_remove_catalog_entry(G, swicol, e, ofd, NULL);
 		if (ret < 0) {
 			/* internal error */
 			E_DEBUG("remove catalog: internal error");
-			rstatus = 8;
-			retval = -1;
+			rstatus = -12;
+			retval = -14;
 			goto error_out;
 		} else if (ret > 0) {
 			/* script error */
 			E_DEBUG("remove catalog: script error");
-			rstatus = 9;
+			rstatus = 10;
 		}
 	}
 
