@@ -50,7 +50,7 @@ class swExProduct: public swExCat
 
 	swPtrList<swExStruct> * getProductList() { return NULL; }
 	
-	char * getObjectName(void) { return "product"; }
+	char * getObjectName(void) { return SW_A_product; }
 
 	static swExStruct * make_exdist(void) {
 		return new swExProduct("");
@@ -58,9 +58,9 @@ class swExProduct: public swExCat
 	
 	virtual int swobjfile_file_out_of_scope(swMetaData * swmd, int current_level) {
 		if (swmd->get_level() < current_level || 
-			strcmp(swmd->get_keyword(), "fileset") == 0 || 
-			strcmp(swmd->get_keyword(), "vendor") == 0 ||
-			strcmp(swmd->get_keyword(), "subproduct") == 0
+			strcmp(swmd->get_keyword(), SW_A_fileset) == 0 || 
+			strcmp(swmd->get_keyword(), SW_A_vendor) == 0 ||
+			strcmp(swmd->get_keyword(), SW_A_subproduct) == 0
 		)
 			return 1;
 		else
@@ -82,7 +82,7 @@ class swExProduct: public swExCat
 			filesetdef = object->getReferer();
 			if (filesetdef == NULL) { setErrorCode(10801, NULL); return; }
 	
-			if (::strcmp(object->getObjectName(), "fileset") != 0)
+			if (::strcmp(object->getObjectName(), SW_A_fileset) != 0)
 				continue;
 			tag = filesetdef->find(SW_A_tag);
 			if (tag == NULL) { setErrorCode(10802, NULL); return; }
@@ -205,9 +205,9 @@ class swExProduct: public swExCat
 	int doesHaveControlFiles(char * key) { return (strcmp(key, "control_file") == 0); }
 
 	int doesContain(char * key) { return (
-					strcmp(key, "fileset") == 0 || 
-					strcmp(key, "subproduct") == 0 ||
-					strcmp(key, "vendor") == 0
+					strcmp(key, SW_A_fileset) == 0 || 
+					strcmp(key, SW_A_subproduct) == 0 ||
+					strcmp(key, SW_A_vendor) == 0
 					);
 				}
 
