@@ -7,7 +7,7 @@
  * COPYING TERMS AND CONDITIONS
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,15 +17,20 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  */
 
 #ifndef swcatalogfile_h_0923
 #define swcatalogfile_h_0923
 
+extern "C" {
+#include "swuser_config.h"
+}
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <limits.h>
 #include "swmemfile.h"
 #include "swpackagefile.h"
 
@@ -121,11 +126,11 @@ class swCatalogFile: public swPackageFile
 	}
 
 
-	virtual int xFormat_write_file(void) {
-		int ret;
+	virtual intmax_t xFormat_write_file(void) {
+		intmax_t ret;
 		int source_fd;
 		char * name;
-		int size;
+		off_t size;
 
 		name = swfile_get_package_filename();	
 		source_fd = swfile_open_public_image_fd();
@@ -135,8 +140,8 @@ class swCatalogFile: public swPackageFile
 			exit(33);
 		}
 
-		size = (int)uxfio_lseek(source_fd, 0L, SEEK_CUR);
-		xFormat_set_filesize(size);
+		size = uxfio_lseek(source_fd, 0L, SEEK_CUR);
+		xFormat_set_filesize((intmax_t)size);
 		xFormat_set_filetype_from_tartype(REGTYPE);
 		xFormat_set_name(name);
 		xFormat_set_nlink(1);
