@@ -195,7 +195,6 @@ void swDefinition::list_replace_if_not_explicitly_set(swMetaData * swmd)
 	}
 }
 
-
 void swDefinition::list_replace(swMetaData * swmd)
 {
 	char * keyword = swmd->get_keyword();
@@ -514,8 +513,74 @@ swDefinition::set_path_attribute(char * source)
 	add(SW_A_path, source);
 }
 
+void 
+swDefinition::squash_duplicates(void)
+{
+	while(i_squash_duplicates()) { ; }
+}
+
+/* Private */
+
+int 
+swDefinition::i_squash_duplicates(void)
+{
+	int retval;
+	int value_type;
+	int ino1;
+	int ino2;
+	char * keyword1;
+	char * keyword2;
+	char * object_keyword;
+	swMetaData * p1;
+	swMetaData * p2;
+	swstructdef defs;
+
+	retval = 0;
+	object_keyword = get_keyword();
+
+	p1 = this;
+	while ((p1 = p1->get_next_node()) != NULL) {
+		ino1 = p1->get_ino();
+		keyword1 = p1->get_keyword();
+		//
+		// Check if attribute is single_value type
+		//
+		if (p1->get_status()) {
+			//
+			// skip attributes that are
+			// already (virtually) removed 
+			//
+			continue;
+		}
+		defs.return_entry_index(object_keyword, keyword1);
+		value_type =  defs.get_value_type();
+		if (value_type < 0) {
+			//
+			// If the attribute is unknown assume it is
+			// a single value type.
+			//
+			value_type = sdf_single_value;
+		}
+		if (value_type != sdf_single_value) {
+			continue;
+		}
+		p2 = this;
+		while ((p2 = p2->get_next_node()) != NULL) {
+			keyword2 = p2->get_keyword();
+			ino2 = p2->get_ino();
+			if (p2->get_status()) {
+				continue;
+			}
+			if (::strcmp(keyword1, keyword2) == 0 && ino1 != ino2) {
+				p1->vremove();
+				retval = 1;
+				break;
+			}
+		}
+	}
+	return retval;
+}
 
-// ----- Private Functions -----
 
 void swDefinition::insert_before_location(swMetaData * swmd, swMetaData * location)
 {
