@@ -133,14 +133,20 @@ swDefinitionFile::generateDefinitions(void) {
 				// seek back because generate_attribute_list() must read this
 				// same line again.
 				//
+				E_DEBUG("making type E");
 				oldoffset = uxfio_lseek(get_mem_fd(), 0, SEEK_CUR);
 				uxfio_lseek(get_mem_fd(), offset, SEEK_SET);
 				generate_attribute_list(u_level, &retval);
 				uxfio_lseek(get_mem_fd(), oldoffset, SEEK_SET);
 			} else if (u_type == 'O') {
+				E_DEBUG("making type O");
 				swdef = makeDefinition(parser_line, &retvalDef);
 				assert(swdef);
 				swdeffile_list_add(swdef); 
+				#ifdef FILENEEDDEBUG
+					fprintf(stderr, "HERE ********************\n");
+					swdef->write_fd(2);
+				#endif
 			} else {
 				retval--;	
 				fprintf (stderr,"error in generateDefinitions(): Unrecognized attribute type.\n");
@@ -440,10 +446,13 @@ swDefinitionFile::swdeffile_linki_write_fd(int fd) {
 	swDefinition *swmd = headM;
 
 	while (swmd) {
+		E_DEBUG("running: determine_status_for_writing(swmd)");
 		do_write = determine_status_for_writing(swmd);
 		if (do_write) {
+			E_DEBUG("do_write is true");
 			eret = swmd->write_fd(fd); 
 		} else {
+			E_DEBUG("do_write is false");
 			eret = 0;
 		}
 		if (eret < 0) return -ret;
@@ -456,21 +465,26 @@ swDefinitionFile::swdeffile_linki_write_fd(int fd) {
 int
 swDefinitionFile::determine_status_for_writing(swDefinition * swmd) { 
 	int ret;
+	int ret2;
+	E_DEBUG("Entering");
 	if (swmd->get_storage_status() == SWDEF_STATUS_DUP0) {
 		// there is a duplicate later on, go find it
 		// and merge it with swmd
-		if (merge_duplicates(swmd)) {
+		E_DEBUG("status is SWDEF_STATUS_DUP0, merging");
+		ret2 = merge_duplicates(swmd);
+		if (ret2 != 0) {
 			//
 			// error
 			//
 			fprintf(stderr,
-				"%s: fatal error merging file definitions, possibly, source attributes are different.\n",
-				swlib_utilname_get());
+				"%s: fatal error merging file definitions, status code: %d\n",
+				swlib_utilname_get(), ret2);
 			exit(47);
 		}
 		ret = 1;
 	} else if (swmd->get_storage_status() == SWDEF_STATUS_DUP) {
 		// skip it 
+		E_DEBUG("status is SWDEF_STATUS_DUP, skip it");
 		ret = 0;
 		;
 	} else {
@@ -491,23 +505,28 @@ swDefinitionFile::merge_duplicates(swDefinition * relhead) {
 	char * isource;
 	char * source;
 
+	E_DEBUG("Entering");
 	swmd = relhead;
 	ipath = swmd->find(SW_A_path);
 	isource = swmd->find(SW_A_source);
 	if (!ipath) return -1;
-	if (!isource) return -1;
+	if (!isource) return -2;
 			
 	swmd = swmd->get_next();
 	while (swmd) {
+		E_DEBUG("New Loop");
 		if (swmd->get_storage_status() == SWDEF_STATUS_DUP) {
 			path = swmd->find(SW_A_path);
-			if (!path) return -1;
+			if (!path) return -3;
 			if (
 				strcmp(path, ipath) == 0
 			) {
+				E_DEBUG2("Got match to merge for path [%s]", path);
 				source = swmd->find(SW_A_source);
-				if (!source) return -1;
-				if (strcmp(source, isource) == 0) {
+				if (!source) return -4;
+				E_DEBUG3("source=[%s] isource=[%s]", source, isource);
+				if (swlib_dir_compare(source, isource, 0) == 0) {
+					E_DEBUG("calling merge");
 					relhead->merge(swmd,
 						0, /* 0 =: replace only attributes
 						      explicitly set */
@@ -517,7 +536,7 @@ swDefinitionFile::merge_duplicates(swDefinition * relhead) {
 					//
 					// Changing the source is not supported
 					//
-					return -2;
+					return -5;
 				}
 			}
 		}
@@ -561,4 +580,3 @@ int swDefinitionFile::open_parser_common(int ifd, int ofd, int filesize) {
 	}
 	return 0;
 }
-
