@@ -49,6 +49,10 @@ extern "C" {
 #include "debug_config.h"
 }
 
+#define SWMETADATANEEDDEBUG 1
+#undef SWMETADATANEEDDEBUG 
+
+
 #ifdef SWMETADATANEEDDEBUG
 #define SWMETADATA_DEBUG(format) SWBISERROR("SWMETADATA DEBUG: ", format)
 #define SWMETADATA_DEBUG2(format, arg) SWBISERROR2("SWMETADATA DEBUG: ", format, arg)
