@@ -113,17 +113,17 @@ int swStruct_i::compare_tag (char * tag) {
 }
 
 swStruct * swStruct_i::swstructure_factory(char * object_keyword) {
-             if ( !::strcmp ("file",object_keyword )) { return swsFile::make_swstructure(); }
-        else if ( !::strcmp ("category",object_keyword )) { return swsCategory::make_swstructure(); }
-        else if ( !::strcmp ("control_file",object_keyword )) { return swsControlFile::make_swstructure(); }
-        else if ( !::strcmp ("fileset",object_keyword )) { return swsFileset::make_swstructure(); }
-        else if ( !::strcmp ("product",object_keyword )) { return swsProduct::make_swstructure(); }
-        else if ( !::strcmp ("subproduct",object_keyword )) { return swssubProduct::make_swstructure(); }
-        else if ( !::strcmp ("distribution",object_keyword )) { return swsDistribution::make_swstructure(); }
-        else if ( !::strcmp ("media",object_keyword )) { return swsMedia::make_swstructure(); }
-        else if ( !::strcmp ("bundle",object_keyword )) { return swsBundle::make_swstructure(); }
-        else if ( !::strcmp ("host",object_keyword )) { return swsHost::make_swstructure(); }
-        else if ( !::strcmp ("vendor",object_keyword )) { return swsVendor::make_swstructure(); }
+             if ( !::strcmp (SW_A_file, object_keyword )) { return swsFile::make_swstructure(); }
+        else if ( !::strcmp (SW_A_category, object_keyword )) { return swsCategory::make_swstructure(); }
+        else if ( !::strcmp (SW_A_control_file, object_keyword )) { return swsControlFile::make_swstructure(); }
+        else if ( !::strcmp (SW_A_fileset, object_keyword )) { return swsFileset::make_swstructure(); }
+        else if ( !::strcmp (SW_A_product, object_keyword )) { return swsProduct::make_swstructure(); }
+        else if ( !::strcmp (SW_A_subproduct, object_keyword )) { return swssubProduct::make_swstructure(); }
+        else if ( !::strcmp (SW_A_distribution, object_keyword )) { return swsDistribution::make_swstructure(); }
+        else if ( !::strcmp (SW_A_media, object_keyword )) { return swsMedia::make_swstructure(); }
+        else if ( !::strcmp (SW_A_bundle, object_keyword )) { return swsBundle::make_swstructure(); }
+        else if ( !::strcmp (SW_A_host, object_keyword )) { return swsHost::make_swstructure(); }
+        else if ( !::strcmp (SW_A_vendor, object_keyword )) { return swsVendor::make_swstructure(); }
         //else if ( !::strcmp ("installed_software",object_keyword )) { return swsInstalled_Software::make_swstructure(); }
         else { return NULL; }
 }
