@@ -139,7 +139,7 @@ swlib_ptym_open(char *pts_name, int * start)
 
 static
 int
-swlib_ptys_open(int fdm, char *pts_name)
+swlib_ptys_open(/*int fdm,*/ char *pts_name)
 {
 	struct passwd	*pw;
 	struct group	*grp;
@@ -252,7 +252,7 @@ swlib_pty_fork2(int *ptrfdm, int ofd, int ifd, int efd,
 				return -1;
 			}
 	
-			if ( (testfds = swlib_ptys_open(fdm, pts_name)) < 0) {
+			if ( (testfds = swlib_ptys_open(/*fdm,*/ pts_name)) < 0) {
 				close(fdm);
 				if (swlib_get_verbose_level() >= SWC_VERBOSE_7) {
 					fprintf(stderr, "%s: can't open slave pty %s\n", 
@@ -305,7 +305,7 @@ swlib_pty_fork2(int *ptrfdm, int ofd, int ifd, int efd,
 			/*
 			 * This is for systems that don't have openpty()
 			 */
-			if ( (fds = swlib_ptys_open(fdm, pts_name)) < 0) {
+			if ( (fds = swlib_ptys_open(/*fdm,*/ pts_name)) < 0) {
 				close(fdm);
 				fprintf(stderr, "can't open slave pty %s : %s\n",
 						pts_name,
