@@ -2,7 +2,7 @@
  */
 
 /*
-   Copyright (C) 1998-2005,2007  James H. Lowe, Jr.
+   Copyright (C) 1998-2005,2007,2011  James H. Lowe, Jr.
    All Rights Reserved.
   
    COPYING TERMS AND CONDITIONS
@@ -67,21 +67,62 @@ static uintmax_t * g_pstatbytes = NULL;
 static struct timespec * io_req = (struct timespec*)NULL;
 static int g_burst_adjust = 65000;
 
-#define TEE_DB_FILE "/tmp/synct_buf.bin"
-#define DO_TEE 1	/* this is for debugging */
-#define DO_TEE 0	/* this is for debugging */
-	
 static
 ssize_t
-synct_i_read(SYNCT * synct, int fd, char * buf)
+synct_i_no_null_write(int fd, char * buf, int sz)
+{
+	int ret;
+	int remains;
+	int excl_region;
+	int amount;
+	int retval;
+	char * p;
+	char * x;
+
+	remains = sz;
+	p = buf;
+	retval = 0;
+	
+	/* buf assumed to be and is NUL terminated */
+	while (remains > 0) {
+		amount = 0;
+		x = p;
+	
+		/* find the length of potentially not-NUL terminated region */
+		while((int)(x - buf) < sz && *x) {
+			amount++;
+			x++;
+		}
+		if (amount > sz) amount = sz;
+		excl_region = remains - amount;
+		E_DEBUG3("amount=%d excl_region=%d", amount, excl_region);
+		ret = atomicio(uxfio_write, fd, p, amount);
+		if (ret < 0) return -1;
+		if (ret != amount) return -2;
+		p += ret;
+		remains -= ret;
+		retval += ret;
+		while (excl_region > 0 && *p == '\0') {
+			excl_region--;
+			remains--;
+			p++;
+		}
+	}
+	return retval;
+}
+ 	
+static
+ssize_t
+synct_i_read_block(SYNCT * synct, int fd, char * buf)
 {
 	ssize_t ret;
-	E_DEBUG("");
+	E_DEBUG2("fd=%d", fd);
 	ret = uxfio_sfa_read(fd, buf, SWLIB_SYNCT_BLOCKSIZE);
 	E_DEBUG("");
 	if (ret != SWLIB_SYNCT_BLOCKSIZE) {
 		fprintf(stderr, "%s: synct_i_read() error: ret=%d\n", swlib_utilname_get(), ret);
-		ret = -1;
+		if (ret < 0) return -1;
+		return -2;
 	}
 	E_DEBUG("");
 	synct->countM++;
@@ -89,56 +130,41 @@ synct_i_read(SYNCT * synct, int fd, char * buf)
 }
 
 static
-char * 
-synct_look_for_eof(SYNCT * synct, unsigned char ** p_trailer_start)
+int
+synct_check_for_possible_eof(SYNCT * synct)
 {
 	unsigned char * ibuf;
 	unsigned char * pos;
-	int len;
+	int i;
 
-	len = synct->buf_lenM;
 	ibuf = synct->bufM;
 	pos = ibuf;
+	i = 0;
 
 	E_DEBUG("");
-	synct->mt_spanM++;
 	while (
-		(int)(pos - ibuf) < len &&
-		*(synct->mtpM) != '\0'
+		(int)(pos - ibuf) < SWLIB_SYNCT_BLOCKSIZE &&
+		synct->mtM[i] != '\0'
 	) {
-		if (*pos && *pos == *(synct->mtpM)) {
+		if (*pos && *pos == synct->mtM[i]) {
 			/* so far so good */
-			E_DEBUG("so far so good");
-			if (*(synct->mtpM) == synct->mtM[0]) {
-				*p_trailer_start = pos;
-				synct->trailer_startM = (char*)pos;
-				synct->mt_spanM = 0;
-			}
-			synct->mtpM++;
-		} else if (*pos == '\0') {
-			/* skip intervening NULs */
-			E_DEBUG("skipping NULs");
-			synct->mt_nullsM++;
-			;
+			E_DEBUG("finding trailer string, so far so good");
+			i++;
 		} else {
 			/* reset, bad match */
-			E_DEBUG("reset, bad match");
-			synct->mt_spanM = 0;
-			synct->trailer_startM = NULL;
-			synct->mtpM = synct->mtM;
-			synct->mt_nullsM = 0;
-			*p_trailer_start = NULL;
+			E_DEBUG("reset, bad match of trailer string");
+			return 0;
 		}
 		pos++;
 	}
 
-	if (*(synct->mtpM) == '\0') {
+	if (synct->mtM[i] == '\0') {
 		/* Found the trailer string */
 		E_DEBUG("found trailer");
-		return (char*)pos;
+		return 1;
 	} else {
 		E_DEBUG("not found");
-		return NULL; /* not found */
+		return 0;
 	}
 }
 
@@ -1543,32 +1569,28 @@ swlib_synct_create(void)
 	SYNCT * synct = (SYNCT *)malloc(sizeof(SYNCT));
 	int len;
 	len = (3 * SWLIB_SYNCT_BLOCKSIZE) + 1;
-	synct->done_posM = -1;
-	synct->doneM = 0;
-	synct->firstM = 1;
-	synct->trailer_startM = NULL;
-	synct->buf_lenM = len;
-	synct->bufM = (unsigned char *)malloc(len);
+	synct->bufM = (unsigned char *)malloc((size_t)len);
+	if (!synct->bufM) _exit(44);
 	memset(synct->bufM, (int)('\0'), len);
 	synct->tailM =  synct->bufM + SWLIB_SYNCT_BLOCKSIZE;
 	synct->countM = 0;
-	synct->remainsM = 0;
-	synct->mt_nullsM = 0;
         synct->mtM = strdup(SWBIS_SYNCT_EOF);
-        synct->mtpM = synct->mtM;
-	synct->mt_spanM = 0;
 	return synct;
 }
 
 void
 swlib_synct_delete(SYNCT * synct)
 {
-        free(synct->mtM);
+	if (synct->countM % 2) {
+		/* if block count is odd give a warning */
+		fprintf(stderr, "%s: Warning: swlib_synct_delete() block count error: %d\n",
+			swlib_utilname_get(), synct->countM);
+	}
+	free(synct->mtM);
 	free(synct->bufM);
 	free(synct);
 }
 
-
 /**
  * swlib_synct_read - read an ascii stream with an in-band EOF string
  * @fd: file descriptor 
@@ -1580,182 +1602,91 @@ swlib_synct_delete(SYNCT * synct)
  * SWLIB_SYNCT_BLOCKSIZE size blocks is always even.  The stream may
  * contain interspersed NULs which are assumed to be not part of the
  * file.
+ * The EOF string (TRAILER!!!\r\n\r\n) must begin a 2*SWLIB_SYNCT_BLOCKSIZE
+ * size block and be entirely NUL filled after.
+ * 
+ * The size of void * userbuf must be 2*SWLIB_SYNCT_BLOCKSIZE or more.
+ * 
+ * The routine will return SWLIB_SYNCT_BLOCKSIZE, or 2*SWLIB_SYNCT_BLOCKSIZE
+ * or 0, or -1
  */
 
 int
-swlib_synct_read(SYNCT * synct, int fd, void * buf)
+swlib_synct_read(SYNCT * synct, int fd, void * userbuf)
 {
 	unsigned char * ibuf;
 	unsigned char * tbuf;
-	char * pos;
-	unsigned char * provisional_trailer_start;
+	char * z;
 	int ret;
-	int tmpret;
-	int rereads;
+	int check_ret;
 
 	ibuf = synct->bufM;
 	tbuf = synct->tailM;	
 
 	/* The output end (sender) must be sync'ed to a block size of
 	   two times SWLIB_SYNCT_BLOCKSIZE */
-		
-	E_DEBUG("begin");
-	if (synct->doneM == 1) {
-		/* Do the shutdown sequence
-		   first step */
-		E_DEBUG("in done step 1");
-		ret = 0;
-		if (synct->remainsM > 0) {
-			E_DEBUG("have remains");
-			memcpy(buf, tbuf, synct->remainsM);
-			ret = synct->remainsM;
-		}
-		/* If the count thus far is odd, then read the final block,
-		   because the sender always sends an even number */
-		E_DEBUG("");
-		if (synct->countM % 2) {
-			E_DEBUG("odd block, read again");
-			tmpret = synct_i_read(synct, fd, (char*)ibuf);
-			if (tmpret < 0) {
-				return -101;
-			}
-			if (DO_TEE) swlib_tee_to_file(TEE_DB_FILE, -1, (char*)ibuf, SWLIB_SYNCT_BLOCKSIZE, 1);
-		}
-		synct->doneM = 2;
-		E_DEBUG("out of done");
-		E_DEBUG2("ret=%d", ret);
-		return ret;
-	} else if (synct->doneM == 2) {
-		/* Do the last step in
-		   shutdown sequence */
-		E_DEBUG("in done step 2");
-		ret = 0;
-		E_DEBUG2("ret=%d", ret);
-		return ret;
-	}
-
+	
+	E_DEBUG2("fd = %d", fd);
+	ret = synct_i_read_block(synct, fd, (char*)ibuf);
+	E_DEBUG2("synct_i_read_block() returned %d", ret);
+	if (ret < 0) return -1;
+	check_ret = synct_check_for_possible_eof(synct);
 	E_DEBUG("");
-	if (synct->firstM == 1) {
-
-		/* First time reading the stream is a
-		   a special case */
-
-		E_DEBUG("first time");
-		E_DEBUG("reading");
-		ret = synct_i_read(synct, fd, (char*)ibuf);
-		if (ret < 0) {
-			return 200 - ret;
-		}
-	 	if (DO_TEE) swlib_tee_to_file(TEE_DB_FILE, -1, (char*)ibuf, SWLIB_SYNCT_BLOCKSIZE, 1);
-		synct->firstM = 0;
-		E_DEBUG("");
+	if (check_ret == 0) {
+		/* EOF not found */
+		memcpy(userbuf, ibuf, ret);
+		return (int)SWLIB_SYNCT_BLOCKSIZE;
 	} else {
-		E_DEBUG("");
-		memcpy(ibuf, tbuf, SWLIB_SYNCT_BLOCKSIZE);
-		memset(tbuf, '\0', SWLIB_SYNCT_BLOCKSIZE+1);
-	}
-
-	E_DEBUG("reading , in mainline");
-	ret = synct_i_read(synct, fd, (char*)tbuf);
-
-	E_DEBUG("");
-	if (ret <= 0) {
-		E_DEBUG2("ret=%d",ret);
-		return -300;
-	}
-	if (DO_TEE) swlib_tee_to_file(TEE_DB_FILE, -1, (char*)tbuf, SWLIB_SYNCT_BLOCKSIZE, 1);
-
-	E_DEBUG("look for eof, in mainline");
-
-	/* Now look for TRAILER!!! in the data */
-
-	do {
-		rereads = 0;
-		provisional_trailer_start = NULL;
-		if (
-			(pos=synct_look_for_eof(synct, &provisional_trailer_start)) != NULL
-		) {
-			/* Found terminating string */
-
-			E_DEBUG("");
-			synct->doneM = 1;
-			synct->done_posM = (int)((char*)synct->trailer_startM - (char*)ibuf);
-			synct->remainsM = 0;
-			E_DEBUG2("found trailer at pos=%d", synct->done_posM);
-			if (synct->done_posM < SWLIB_SYNCT_BLOCKSIZE) {
-				E_DEBUG("");
-				ret = synct->done_posM;
-			} else if (synct->done_posM >= SWLIB_SYNCT_BLOCKSIZE) {
-				E_DEBUG("");
-				ret = SWLIB_SYNCT_BLOCKSIZE;
-				synct->remainsM = synct->done_posM - SWLIB_SYNCT_BLOCKSIZE;
-			} else {
-				;
-			}
-			E_DEBUG("");
-			memcpy(buf, ibuf, ret);
-			break;
-		} else if (provisional_trailer_start) {
-			/* Found part of terminating string, we must now determine what
-			   it really is by continuing to read */
-
-			E_DEBUG("reading again while looking for the trailer");
-		
-			/* This handles the case where the string lands on a block boundary *and*
-			   has an entire NUL block between the two blocks */
-
-			rereads++;
-			ret = synct_i_read(synct, fd, (char*)((char*)tbuf+SWLIB_SYNCT_BLOCKSIZE));
-			if (ret <= 0) {
-				return -400;
-			}
+		/* EOF found */
+		/* read 1 more block which should be NUL
+		   if so its the trailer block therefore
+		   return 0 bytes to the user */
+		E_DEBUG2("fd = %d", fd);
+		ret = synct_i_read_block(synct, fd, (char*)tbuf);
+		if (ret < 0) return -2;
+		z = malloc((size_t)SWLIB_SYNCT_BLOCKSIZE);
+		memset(z, '\0', (size_t)SWLIB_SYNCT_BLOCKSIZE);
+		ret = memcmp(z, tbuf, SWLIB_SYNCT_BLOCKSIZE);
+		free(z);
+		if (ret == 0) {
+			/* really EOF */
+			return 0;
 		} else {
-			/* This code block handles the case when there is a false match
-			   of the TRAILER string */
-
-			E_DEBUG("");
-			synct->doneM = 0;
-			if (rereads > 1) {
-				/* fatal, unhandled */
-				E_DEBUG("");
-				return -404;
-			}
-			if (rereads == 0) {
-				E_DEBUG("");
-				ret = SWLIB_SYNCT_BLOCKSIZE;
-				memcpy(buf, ibuf, ret);
-			} else {
-				/* reread can only equal 1 here */
-				E_DEBUG("");
-				ret = SWLIB_SYNCT_BLOCKSIZE + (rereads * SWLIB_SYNCT_BLOCKSIZE);
-				memcpy(buf, ibuf, ret);
-				memcpy(ibuf, tbuf+SWLIB_SYNCT_BLOCKSIZE, SWLIB_SYNCT_BLOCKSIZE);
-			}
-			break;
+			/* we've been fooled !! */
+			/* we now have read two (2) blocks which must be 
+			   returned to the user */
+			memcpy(userbuf, ibuf, SWLIB_SYNCT_BLOCKSIZE);
+			memcpy(((char*)userbuf)+SWLIB_SYNCT_BLOCKSIZE, tbuf, SWLIB_SYNCT_BLOCKSIZE);
+			return (int)(2*SWLIB_SYNCT_BLOCKSIZE);	
 		}
-		E_DEBUG("");
-	} while (provisional_trailer_start != NULL);
-
-	E_DEBUG2("ret=%d", ret);
-	return ret;
+	}
+	/* never gets here */
+	return -2;
 }
 
+/**
+ * swlib_synct_suck - rewrites a stream containing an in-band EOF string
+ * The stream is written by dd sync=conv to 2*SWLIB_SYNCT_BLOCKSIZE size blocks
+ * NULs '\0' are assumed to be not part of the data.  The trailer string
+ * must begin a 2*SWLIB_SYNCT_BLOCKSIZE size block and be filled with NULs
+ */
+
 int
 swlib_synct_suck(int ofd, int ifd)
 {
 	int ret;
 	int wret;
 	int wtotal = 0;
-	char buf[SWLIB_SYNCT_BLOCKSIZE+SWLIB_SYNCT_BLOCKSIZE];
+	char buf[SWLIB_SYNCT_BLOCKSIZE+SWLIB_SYNCT_BLOCKSIZE + 1];
 	SYNCT * synct;
-
-	if (DO_TEE) swlib_tee_to_file(TEE_DB_FILE, -1, "", 0, 0);
 	synct = swlib_synct_create();
 	do {
 		ret = swlib_synct_read(synct, ifd, buf);
+		E_DEBUG2("swlib_synct_read returned %d", ret);
 		if (ret > 0) {
-			wret = uxfio_write(ofd, buf, ret);
-			if (wret != ret) {
+			wret = synct_i_no_null_write(ofd, buf, ret);
+			if (wret < 0) {
+				E_DEBUG("error");
 				return -2;
 			}	
 			wtotal += wret;
@@ -2114,6 +2045,7 @@ swlib_kill_all_pids(pid_t * pid, int num, int signo, int verbose_level)
 				/* (struct sw_logspec *) */(NULL), STDERR_FILENO,
 				"swlib_kill_all_pids: kill[%d] signo=%d\n",
 							(int)pid[i], signo);
+			E_DEBUG3("killing pid %d with signal %d", (int)pid[i], signo);
 			if (kill(pid[i], signo) < 0) {
 				swlib_doif_writef(verbose_level,  SWC_VERBOSE_SWIDB,
 					/* (struct sw_logspec *)*/ (NULL), STDERR_FILENO,
@@ -2141,21 +2073,45 @@ swlib_update_pid_status(pid_t keypid, int value, pid_t * pid, int * status, int
 	}
 	return -1;
 }
-
 int
 swlib_wait_on_all_pids(pid_t * pid, int num, int * status, 
 				int flags, int verbose_level)
 {
+	return
+		swlib_wait_on_all_pids_with_timeout(pid, num, status, flags, verbose_level, 0);
+}
+
+int
+swlib_wait_on_all_pids_with_timeout(pid_t * pid, int num, int * status, 
+				int flags, int verbose_level, int fp_tmo)
+{
+	time_t now;
+	time_t start;
 	int wret;
 	int done = 0;
 	int i = 0;
 	int got_one = 0;
+	int tmo;
+	int do_kill;
+
+	do_kill = 0;
+	if (fp_tmo < 0) {
+		do_kill = 1;
+		tmo = -fp_tmo;
+	} else {
+		tmo = fp_tmo;
+	}
+
 	if (num == 0) return 0;
 	E_DEBUG("");
-	while (!done) {
+	start = time(NULL);
+	now = start;
+
+	while (!done && (tmo == 0 || (int)(now - start) < tmo)) {
 		done = 1;
 		/* usleep(100000); */ /* Sleep 1/10th second, this greatly */
 				      /* improves performance on Cygwin    */
+		now = time(NULL);
 		E_DEBUG("");
 		for(i=0; i<num; i++) {
 			if (pid[i] > 0) {
@@ -2187,8 +2143,20 @@ swlib_wait_on_all_pids(pid_t * pid, int num, int * status,
 					got_one = 1;
 					pid[i] = -pid[i];
 				}
+			} else {
+				E_DEBUG2("already processed: pid=%d", (int)pid[i]);
+				;
 			}
 		}
+		if (flags == WNOHANG)
+			done = 1;
+	}
+	if (do_kill) {
+		E_DEBUG("killing pids SIGINT");
+		swlib_kill_all_pids(pid, num, SIGINT, 3);
+		sleep(1);
+		E_DEBUG("killing pids SIGKILL");
+		swlib_kill_all_pids(pid, num, SIGKILL, 3);
 	}
 	return status[num-1];
 }
@@ -2791,11 +2759,21 @@ void
 swlib_append_synct_eof(STROB * buf)
 {
 	/* Here is the SWBIS_SYNCT_EOF */
+	/*  Here's how to make it:  printf 'TRAILER!!!\r\n\r\n' | dd conv=synv bs=256 */
+	E_DEBUG("");
+
 	strob_sprintf(buf, 1, /* append */
-		"printf \"" CPIO_INBAND_EOA_FILENAME "\\r\\n\"\n"
-		"printf \"\\r\\n\"\n"
-		") | dd 2>/dev/null\n"
-		") | dd ibs=%d obs=%d 2>/dev/null | dd ibs=%d obs=%d conv=sync 2>/dev/null\n",
-		SWLIB_SYNCT_BLOCKSIZE*3, SWLIB_SYNCT_BLOCKSIZE*3, SWLIB_SYNCT_BLOCKSIZE*2, SWLIB_SYNCT_BLOCKSIZE*2);
-		/* Ending Subshell with sync'ing dd */
+		") | dd ibs=%d obs=%d 2>/dev/null | dd ibs=%d obs=%d conv=sync 2>/dev/null |\n"
+		"	(\n"
+		"		dd bs=%d 2>/dev/null\n" 
+		"		( printf \"" CPIO_INBAND_EOA_FILENAME "\\r\\n\"\n" 
+		"		printf \"\\r\\n\" ) | dd ibs=%d obs=%d conv=sync 2>/dev/null\n"
+		"	)\n"
+		") | dd ibs=%d obs=%d conv=sync 2>/dev/null\n",
+		SWLIB_SYNCT_BLOCKSIZE*3, SWLIB_SYNCT_BLOCKSIZE*3, SWLIB_SYNCT_BLOCKSIZE*2, SWLIB_SYNCT_BLOCKSIZE*2,
+		SWLIB_SYNCT_BLOCKSIZE*2,
+		SWLIB_SYNCT_BLOCKSIZE*2, SWLIB_SYNCT_BLOCKSIZE*2,
+		SWLIB_SYNCT_BLOCKSIZE*2, SWLIB_SYNCT_BLOCKSIZE*2
+		);
+	return;
 }
