@@ -79,6 +79,6 @@
    int		swsdflt_get_value_type 		(struct swsdflt_defaults * swd);
    /* char *	swsdflt_get_permitted_value 	(struct swsdflt_defaults * swd); */
    int 		swsdflt_get_has_default_value 	(struct swsdflt_defaults * swd);
-   char *	swsdflt_get_default_value 	(struct swsdflt_defaults * swd);
-
+   char *	swsdflt_get_default_value_by_ent  (struct swsdflt_defaults * swd);
+   char *       swsdflt_get_default_value        (char * object, char * keyword);
 #endif
