@@ -54,9 +54,10 @@ printlogline(STROB * buf, char * sptr)
 	time_t tm;
 	char result[64];
 	char * s;
-	
 	time(&tm);
-	ctime_r(&tm, result);
+	/*ctime_r(&tm, result); */
+        strncpy(result, asctime(localtime(&tm)), sizeof(result) - 1);
+        result[sizeof(result)-1] = '\0';
 	s = result;
 	s = strchr(result, ' ');
 	if (s) {
