@@ -260,7 +260,7 @@ handle_trailing_slash(char * mode, char * name, int *pflag)
 
 static
 int 
-filename_does_match(SWVARFS * swvarfs, char * name, char * candidate)
+filename_does_match(char * name, char * candidate)
 {
 	int ret;
 	ret = swlib_compare_8859(name, candidate);
@@ -548,7 +548,7 @@ set_linkname(SWVARFS * swvarfs, int ufd, char * path)
 	s[c] = '\0';	
 	if (strlen(s) > 99) {
 		fprintf(stderr, 
-		"Warning, linkname [%s] is too long for tar archives.\n", s);
+		"Warning, linkname [%s] is too long for tar archives\n", s);
 	}
 	if (vlinkp){ 
 		swbis_free(vlinkp); 
@@ -607,21 +607,103 @@ swvarfs_i_attach_filechain(SWVARFS * swvarfs, struct fileChain *new)
 
 static
 int
-stat_node(SWVARFS * swvarfs, char * path, struct stat * statbuf)
+stat_node(SWVARFS * swvarfs, char * fp_path, struct stat * statbuf)
 {
-	if (lstat(path, statbuf) < 0) {
-		fprintf(stderr,"getnext_dirent: lstat failed: %s\n", path);
-		swvarfs->derrM = -2;
-		return -1;
+	char * path;
+	int i;
+	STROB * linkbuf;
+	int bufsize;
+	STROB * totallinkbuf;
+	int ret;
+
+	SWVARFS_E_DEBUG("BEGIN"); 
+	bufsize=300;
+	linkbuf = NULL;
+	totallinkbuf = NULL;
+	path = fp_path;
+	i = 0;
+	SWVARFS_E_DEBUG(""); 
+
+	/* ... actually, the stat() system call follows multiple links
+	   so this while loop is probably not required as it will
+	   never loop more that once (on a Linux kernel at least). */
+
+	while (i++ < swvarfs->n_loop_symlinksM) {
+		SWVARFS_E_DEBUG2("in sumlink loop, i=%d", i); 
+		if ((*(swvarfs->f_statM))(path, statbuf) < 0) {
+			fprintf(stderr,"%s: getnext_dirent: %s() failed: %s\n",
+				swlib_utilname_get(), swvarfs_get_stat_syscall(swvarfs), path);
+			swvarfs->derrM = -2;
+			return -1;
+		}
+
+		if (*swvarfs_get_stat_syscall(swvarfs) == 'l') {
+			/* lstat() system call being used */
+			/* If the stat call is 'lstat' only do the loop once */
+			break;
+		} else {
+			/* The stat() system call being used */
+			; 
+		}
+
+		if (S_ISLNK(statbuf->st_mode)) {
+			SWVARFS_E_DEBUG("Never gets here"); 
+			/* This code will never happen on a kernel 
+			   stat() call resolves multiple symbolic links */
+		
+			SWVARFS_E_DEBUG("ISLNK is true"); 
+			if (linkbuf == NULL) {
+				linkbuf = strob_open(bufsize+2);
+			}
+			if (totallinkbuf == NULL) {
+				totallinkbuf = strob_open(bufsize);
+				strob_strcpy(totallinkbuf, path);
+			}
+			strob_memset(linkbuf, (int)('\0'), bufsize+2);
+			ret = readlink(strob_str(totallinkbuf), strob_str(linkbuf), bufsize);
+			SWVARFS_E_DEBUG("");
+			if (ret < 0 || ret >= bufsize-1) {
+				if (ret < 0) {
+					fprintf(stderr,"%s: readlink failed: %s on %s\n",
+						swlib_utilname_get(), strob_str(totallinkbuf), strerror(errno));
+				} else {
+					fprintf(stderr,"%s: readlink failed: program buffer: %s\n",
+						swlib_utilname_get(), strob_str(totallinkbuf));
+				}
+				return -1;
+			}
+			SWVARFS_E_DEBUG(""); 
+			if (*strob_str(linkbuf) == '/') {
+				strob_strcpy(totallinkbuf, strob_str(linkbuf));
+			} else {
+				strob_strcat(totallinkbuf, "/");
+				strob_strcat(totallinkbuf, strob_str(linkbuf));
+			}
+			path = strob_str(totallinkbuf);
+		} else {
+			break;
+		}
 	}
+
+	if (linkbuf) strob_close(linkbuf);
+	if (totallinkbuf) strob_close(totallinkbuf);
+
+	if (i >= swvarfs->n_loop_symlinksM) {
+		fprintf(stderr,"%s: getnext_dirent: recursive %s failed on %s\n",
+				swlib_utilname_get(), swvarfs_get_stat_syscall(swvarfs), fp_path);
+			swvarfs->derrM = -2;
+		return -2;
+	}
+
 	taru_statbuf2filehdr(ahs_vfile_hdr(swvarfs->ahsM), statbuf,
 							 NULL, NULL, NULL);
-	ahsStaticSetTarFilename(ahs_vfile_hdr(swvarfs->ahsM), path);
+	ahsStaticSetTarFilename(ahs_vfile_hdr(swvarfs->ahsM), fp_path);
 	if (S_ISLNK(statbuf->st_mode)) {
-		set_linkname(swvarfs, -1, path);
+		set_linkname(swvarfs, -1, fp_path);
 	} else {
 		ahsStaticSetTarLinkname(ahs_vfile_hdr(swvarfs->ahsM), NULL);
 	}
+	SWVARFS_E_DEBUG("END"); 
 	return 0;
 }
 
@@ -884,6 +966,7 @@ swvarfs_i_init(SWVARFS * swvarfs)
 	swvarfs->taruM = taru_create();
 	swvarfs->taruM->taru_tarheaderflagsM = 0;
 	SWVARFS_E_DEBUG2("LOWEST FD=%d", show_nopen()); 
+	swvarfs->n_loop_symlinksM = 20;
 }
 
 static
@@ -990,7 +1073,7 @@ swvarfs_i_u_lstat_fs(SWVARFS * swvarfs, char * path, struct stat * st)
 	} else {
 		strob_strcpy(swvarfs->u_nameM, path);
 	}
-	ret = lstat(strob_str(swvarfs->u_nameM), st);
+	ret = (*(swvarfs->f_statM))(strob_str(swvarfs->u_nameM), st);
 	SWVARFS_E_DEBUG("LEAVING");
 	return ret;
 }
@@ -1377,7 +1460,7 @@ swvarfs_i_u_open_archive_seek_to_name(SWVARFS * swvarfs, char * name)
 	/*if (*peoa != 0) return 0; */
 
 	if (current_name && 
-		!filename_does_match(swvarfs, name, current_name) &&
+		!filename_does_match(name, current_name) &&
 				existing == NULL) {
 		/*
 		* special case for inorder traversal.
@@ -2088,6 +2171,11 @@ swvarfs_setdir(SWVARFS * swvarfs, char * path)
 		int statrc;
 		struct stat stbuf;
 
+	        dirx.statbuf = NULL;
+		dirx.dirp = NULL;
+		dirx.dp = NULL;
+		dirx.ptr = NULL;
+
 		clear_all_dir_context(swvarfs);
 
 		statrc = stat(path, &stbuf);
@@ -2571,6 +2659,19 @@ swvarfs_set_stat_syscall(SWVARFS * swvarfs, char * s)
 	}
 }
 
+char *
+swvarfs_get_stat_syscall(SWVARFS * swvarfs)
+{
+	if ( swvarfs->f_statM == (int (*)(char *, struct stat *))lstat) {
+		return SWVARFS_VSTAT_LSTAT;
+	} else if ( swvarfs->f_statM == (int (*)(char *, struct stat *))stat) {
+		return SWVARFS_VSTAT_STAT;
+	} else {
+		fprintf(stderr, "swvarfs: fatal error in swvarfs_get_stat_syscall\n");
+		exit(1);
+	}
+}
+
 int swvarfs_get_tarheader_flags(SWVARFS * swvarfs) {
 	return swvarfs->taruM->taru_tarheaderflagsM;
 }	
