@@ -166,11 +166,11 @@ error_msg_control(char * name, int id, char * dbname, int * is_new)
 	}
 
 	if (name) {
-		strob_sprintf(tmp, 0, "%s: %s name [%s] not found in system database.\n",
+		strob_sprintf(tmp, 0, "%s: %s name [%s] not found in system database\n",
 					 swlib_utilname_get(), dbname, name);
 	} else {
 		strob_sprintf(tmp, 0,
-			"%s: %s id [%d] not found in system database.\n",
+			"%s: %s id [%d] not found in system database\n",
 				swlib_utilname_get(), dbname, (int)(id));
 	}
 	if (strstr(strob_str(store), strob_str(tmp))) {
