@@ -514,7 +514,7 @@ taru_write_archive_member_header(TARU * taru,
 	int do_strip_leading_slash = 0;
 	int linkrecord_active;
 
-	E_DEBUG("");
+	E_DEBUG("BEGIN");
 	file_hdr = ahsStaticCreateFilehdr();
 	linkrecord_active = (linkrecord != NULL && taru->linkrecord_disableM == 0);
 
@@ -584,8 +584,9 @@ taru_write_archive_member_header(TARU * taru,
 		return -2;
 	}
 
+	E_DEBUG("");
 	if (do_gnu_long_link) {
-		E_DEBUG("");
+		E_DEBUG("in do_gnu_long_link");
 		if ((do_gnu_long_link=taru_write_long_link_member
 				(taru, out_file_des,
 					ahsStaticGetTarFilename(file_hdr),
@@ -659,7 +660,7 @@ taru_write_archive_member_header(TARU * taru,
 	} else if (archive_format_out == arf_ustar ||
 		archive_format_out == arf_tar) {
 		if (linkrecord_active) {
-			   E_DEBUG("");
+			   E_DEBUG("tar format");
 			   nfound=0;	
 			   if (file_hdr->c_nlink > 1) {
 				link_record_buf = hllist_find_file_entry(linkrecord, 
@@ -704,6 +705,7 @@ taru_write_archive_member_header(TARU * taru,
 			is traversed.
 			*/
 		}
+		E_DEBUG("Entering taru_write_out_header");
 		retval = taru_write_out_header
 			(taru, file_hdr, out_file_des, 
 				archive_format_out,
@@ -924,6 +926,7 @@ taru_write_out_header(TARU * taru, struct new_cpio_header *file_hdr,
 	char *magic_string;
 	int do_strip_leading_slash = 0;
 
+	E_DEBUG("BEGIN");
 	if (taru) { 
 		do_strip_leading_slash = 
 			tarheaderflags & TARU_TAR_DO_STRIP_LEADING_SLASH;
@@ -1040,6 +1043,7 @@ taru_write_out_header(TARU * taru, struct new_cpio_header *file_hdr,
 	/* TAR */
 	} else if (archive_format_out == arf_tar ||
 				archive_format_out == arf_ustar) {
+		E_DEBUG("calling taru_write_out_tar_header2");
 		ret = taru_write_out_tar_header2(taru, 
 						file_hdr,
 						out_des,
@@ -1146,7 +1150,6 @@ taru_process_copy_out(TARU * taru, int input_fd, int out_file_des,
 	intmax_t retvalx;
 	HLLIST * hllist;
 	int header_length;
-	char intbuf[UINTMAX_STRSIZE_BOUND];
 
 	E_DEBUG("");
 	file_hdr0 = ahsStaticCreateFilehdr();
