@@ -4,7 +4,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -14,7 +14,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 
 #include "swuser_config.h"
@@ -220,7 +220,7 @@ taru_process_copy_in(int i_fd, int output_fd)
 						nmore = file_hdr->c_nlink - 1;	/* must now go and find nmore occurances of the file
 										   in the cpio archive because thats where the data is. */
 						data_offset = orig_offset;
-						data_size = file_hdr->c_filesize;
+						data_size = (int)(file_hdr->c_filesize);
 						count = 0;
 						found_data = 0;
 						done_links = FALSE;
@@ -363,7 +363,7 @@ taru_process_copy_in(int i_fd, int output_fd)
 			switch (file_hdr->c_mode & CP_IFMT) {
 			case CP_IFREG:
 				taru_write_out_tar_header2(taru, file_hdr, output_fd, (char *)(NULL), (char *)(NULL), (char *)(NULL), 0);
-				taru_pump_amount2(output_fd, input_fd, file_hdr->c_filesize, -1);
+				taru_pump_amount2(output_fd, input_fd, (intmax_t)(file_hdr->c_filesize), -1);
 				taru_tape_skip_padding(input_fd, file_hdr->c_filesize, archive_format_in);
 				taru_tape_pad_output(output_fd, file_hdr->c_filesize, archive_format_tar);
 				break;
@@ -401,8 +401,8 @@ taru_process_copy_in(int i_fd, int output_fd)
 
 		}
 	}
-	write(output_fd, zeros_512, 512);
-	write(output_fd, zeros_512, 512);
+	uxfio_unix_safe_write(output_fd, zeros_512, 512);
+	uxfio_unix_safe_write(output_fd, zeros_512, 512);
 
 
 	if (archive_format_in != arf_ustar) {
