@@ -1245,11 +1245,16 @@ LABEL_REREAD:
 			unzip[0] = shcmd_open();
 			shcmd_add_arg(unzip[0], "bzip2");
 			shcmd_add_arg(unzip[0], "-d");
+		} else if (memcmp(((char*)(xbuffer)) + deb_gz_offset,
+				lzmamagic, LENGTH_OF_MAGIC) == 0) {
+			unzip[0] = shcmd_open();
+			shcmd_add_arg(unzip[0], "lzma");
+			shcmd_add_arg(unzip[0], "-d");
 		} else {
 			/*
 			 * error, the compressed magic not found at the expected offset
 			 */
-			fprintf(stderr, "%s: internal error %s: at line %d\n", swlib_utilname_get(), __FILE__, __LINE__);
+			fprintf(stderr, "%s: unsupported compression in a .deb file %s: at line %d\n", swlib_utilname_get(), __FILE__, __LINE__);
 			return -1;
 		} 
 
