@@ -3,10 +3,9 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "swparser_global.h"
 #include "swpackagefile.h"
 #include "swattributefile.h"
-
+#include "swmain.h"
 int
 main (int argc, char ** argv)
 {
