@@ -5,7 +5,8 @@
 #include <string.h>
 #include <unistd.h>
 #include <typeinfo>
-#include "stream_config.h"
+
+#include "xstream_config.h"
 #include "swpathname.h"
 #include "swmain.h"
 
