@@ -55,7 +55,7 @@ ${TOUCH} -t $create_date tmp
 
 (
 	export SWPACKAGEPASSFD=3
-	exec 3<$PASSFILE swign -u "$AUTHOR" -D bin/checksig.sh -S
+	exec 3<$PASSFILE swign -s . -u "$AUTHOR" -D bin/checkdigest.sh -S
 )
 
 (rm -f tmp/xx; cd tmp; ln -s ../swprogs/xx xx)
